// 004 u_c_standard_maze_builder_2
// 17 oct 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   John COLIBRI  http://www.jcolibri.com

(*$r+*)

unit u_c_standard_maze_builder_2;
  interface
    uses u_c_maze_2
        , u_c_maze_builder_2
        ;

    type c_standard_maze_builder= class(c_maze_builder)
                                    _m_c_current_maze: c_maze;

                                    Constructor create_standard_maze_builder;

                                    procedure build_maze; Override;
                                    procedure build_room(p_room_number: Integer); Override;
                                    procedure build_door(p_from_room_number, p_to_room_number: Integer); Override;

                                    function f_c_maze: c_maze; Override;

                                    procedure _build_the_maze;
                                    function _f_common_wall(p_c_from_room, p_c_to_room: c_room): t_direction;
                                  end; // c_standard_maze_builder

  implementation

    // -- c_standard_maze_builder

    Constructor c_standard_maze_builder.create_standard_maze_builder;
      begin
        Inherited Create;
      end; // create_standard_maze_builder

    procedure c_standard_maze_builder.build_maze;
      begin
        _m_c_current_maze:= c_maze.create_maze;
      end; // build_maze

    procedure c_standard_maze_builder.build_room(p_room_number: Integer);
      var l_c_room: c_room;
      begin
        // -- add another room if none present with the same number
        if _m_c_current_maze.f_c_room(p_room_number)= Nil
          then begin
              l_c_room:= c_room.create_room(p_room_number);
              _m_c_current_maze.add_room(l_c_room);

              with l_c_room do
              begin
                set_side(e_north, c_wall.create_wall);
                set_side(e_east, c_wall.create_wall);
                set_side(e_south, c_wall.create_wall);
                set_side(e_west, c_wall.create_wall);
              end; // with l_c_room
            end;
      end; // build_room

    procedure c_standard_maze_builder.build_door(p_from_room_number, p_to_room_number: Integer);
      var l_c_room_from, l_c_room_to: c_room;
          l_c_door: c_door;
      begin
        l_c_room_from:= _m_c_current_maze.f_c_room(p_from_room_number);
        l_c_room_to:= _m_c_current_maze.f_c_room(p_to_room_number);
        l_c_door:= c_door.create_door(l_c_room_from, l_c_room_to);

        l_c_room_from.set_side(_f_common_wall(l_c_room_from, l_c_room_to), l_c_door);
        l_c_room_to.set_side(_f_common_wall(l_c_room_from, l_c_room_to), l_c_door);
      end; // build_door

    function c_standard_maze_builder.f_c_maze: c_maze;
      begin
        Result:= _m_c_current_maze;
      end; // f_c_maze

    procedure c_standard_maze_builder._build_the_maze;
      begin

      end; // _build_the_maze

    function c_standard_maze_builder._f_common_wall(p_c_from_room, p_c_to_room: c_room): t_direction;
      begin
        // -- not done. Avoid warning
        Result:= e_north;
      end; // _f_common_wall

    begin // u_c_standard_maze_builder
    end. // u_c_standard_maze_builder

